<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

/**
 * FacebookChannelHelper class.
 * AutoTweet Facebook channel for wall posts.
 * Posts to the wall of profiles, groups, pages, events
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class FacebookChannelHelper extends FacebookBaseChannelHelper
{
	/**
	 * sendMessage.
	 *
	 * @param   string  $message  Params
	 * @param   object  $data     Params
	 *
	 * @return	booleans
	 */
	public function sendMessage($message, $data)
	{
		try
		{
			$this->cleanCacheAllIncluded();
		}
		catch (Exception $e)
		{
		}

		// Simulated
		if ($this->channel->params->get('use_own_api') == 0)
		{
			return $this->testMessage();
		}

		$isUserProfile = $this->isUserProfile();
		$enabledOpenGraphFeatures = $this->channel->params->get('open_graph_features');

		if (($enabledOpenGraphFeatures) && ($isUserProfile))
		{
			return $this->sendFacebookOG($message, $data->title, $data->fulltext, $data->url, $data->org_url, $data->image_url, $this->getMediaMode(), $data);
		}

		$name 			= $data->title;
		$description 	= $data->fulltext;
		$url 			= $data->url;
		$org_url 		= $data->org_url;
		$picture 		= $data->image_url;
		$mediaMode 		= $this->getMediaMode();

		$logger = AutotweetLogger::getInstance();

		$postLink 		= $org_url;

		if (($data->plugin == 'autotweetpost') && ($data->id))
		{
			$postLink = 'index.php?option=com_autotweet&view=composerpost&id='
				. $data->id . '-' .
				TextUtil::convertUrlSafe($data->title);

			$needles = array();
			$needles['view'] = 'composerpost';
			$itemId = AutotweetBaseHelper::getItemid('com_autotweet', $needles);

			if ($itemId)
			{
				$postLink = $postLink . '&Itemid=' . $itemId;
			}

			$routeHelp = RouteHelp::getInstance();

			if (RouteHelp::isMultilingual())
			{
				$postLink = $routeHelp->getAbsoluteRawUrl($postLink);
			}
			else
			{
				$postLink = $routeHelp->getAbsoluteUrl($postLink);
			}

			$logger->log(JLog::INFO, 'Composerpost Url: ' . $postLink);
		}

		$logger->log(JLog::INFO, 'sendFacebookMessage', $message);

		$fbId = $this->getFbChannelId();
		$accessToken = $this->get('fbchannel_access_token');
		$accessToken = $this->matchToken($accessToken, $fbId);

		// Post message and/or attachment
		switch ($mediaMode)
		{
			case 'attachment':
				$postAttachment = true;
				$postMessage = false;
				break;
			case 'both':
				$postMessage = true;
				$postAttachment = true;
				break;
			case 'message':
			default:
				$postMessage = true;
				$postAttachment = false;
		}

		$actions = array();
		$caption = null;

		if (empty($org_url))
		{
			$postAttachment = false;
		}
		else
		{
			// Extract data for action link
			$url_comps = parse_url($org_url);
			$caption = $url_comps['host'];

			if (EParameter::getComponentParam(CAUTOTWEETNG, 'include_authors'))
			{
				$author = $data->xtform->get('author');
				$author = JFactory::getUser($author);
				$authorName = $author->name;

				if (!empty($authorName))
				{
					$caption = $caption . ' | ' . $authorName;
				}
			}

			$actions['name'] = $caption;
			$actions['link'] = $postLink;
		}

		$name = TextUtil::truncString($name, self::MAX_CHARS_NAME, false, true);

		$arguments = array(
			'link' => $postLink,
		);

		if (!$this->isApi11OrSuperior)
		{
			$arguments['name'] = $name;
			$arguments['caption'] = $caption;
			$arguments['description'] = $description;
		}

		if (!empty($actions))
		{
			$arguments['actions'] = json_encode($actions);
		}

		if (strpos($org_url, '//www.youtube.com/') != false)
		{
			$source = str_replace('//www.youtube.com/watch?v=', '//www.youtube.com/v/', $org_url);
			$arguments['source'] = $source;

			if (empty($picture))
			{
				$picture = str_replace('//www.youtube.com/watch?v=', '//img.youtube.com/vi/', $org_url) . '/0.jpg';
			}
		}

		if ($isUserProfile)
		{
			$privacy = $this->get('sharedwith', 'EVERYONE');
			$privacy = array('value' => $privacy);
			$arguments['privacy'] = json_encode($privacy);
		}

		if ((empty($description)) && (empty($picture)))
		{
			$postAttachment = false;
		}

		// Include image tag only, when image url is not empty to avoid error "... must have a valid src..."
		if ( (!$this->isApi11OrSuperior) && (!empty($picture)) && ($postAttachment) )
		{
			$arguments['picture'] = $picture;
		}

		// Message
		if ($postMessage)
		{
			$arguments['message'] = $message;
		}

		try
		{
			$logger->log(JLog::INFO, 'FacebookChannelHelper callApiPost', $arguments);

			$post = $this->callApiPost('/' . $fbId . '/feed', $arguments, $accessToken);

			$msg = 'Facebook id: ' . $post['id'] . ' - ' . $postLink;
			$result = array(
				true,
				$msg
			);
		}
		catch (Exception $e)
		{
			$code = $e->getCode();
			$msg = $code . ' - ' . $e->getMessage() . ' - ' . $postLink;

			$result = array(
				false,
				$msg
			);
		}

		return $result;
	}

	/**
	 * cleanCacheAllIncluded
	 *
	 * @return  void
	 */
	protected function cleanCacheAllIncluded()
	{
		$pluginData = JPluginHelper::getPlugin('content', 'autotweetopengraph');

		if (empty($pluginData))
		{
			return;
		}

		$params = json_decode($pluginData->params, true);

		if (empty($params))
		{
			return;
		}

		if (!isset($params['included_components']))
		{
			$params['included_components'] = 'com_content,com_easyblog,com_flexicontent,com_k2,com_zoo';
		}

		$includedComponents = $params['included_components'];
		$includedComponents = explode(',', str_replace(array('\n', ' '), array(',', ''), $includedComponents));

		if (empty($includedComponents))
		{
			return;
		}

		foreach ($includedComponents as $component)
		{
			$this->cleanCache($component);
		}
	}

	/**
	 * Clean the cache
	 *
	 * @param   string   $group      The cache group
	 * @param   integer  $client_id  The ID of the client
	 *
	 * @return  void
	 *
	 * @since   3.0
	 */
	protected function cleanCache($group = null, $client_id = 0)
	{
		$conf = JFactory::getConfig();

		$options = array(
			'defaultgroup' => $group ?: (isset($this->option) ? $this->option : JFactory::getApplication()->input->get('option')),
			'cachebase' => $client_id ? JPATH_ADMINISTRATOR . '/cache' : $conf->get('cache_path', JPATH_SITE . '/cache'),
			'result' => true,
		);

		try
		{
			/** @var JCacheControllerCallback $cache */
			$cache = JCache::getInstance('callback', $options);
			$cache->clean();
		}
		catch (JCacheException $exception)
		{
			$options['result'] = false;
		}

		// Trigger the onContentCleanCache event.
		JEventDispatcher::getInstance()->trigger('onContentCleanCache', $options);
	}
}
