<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

/**
 * FacebookInstantChannelHelper class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class FacebookInstantChannelHelper extends FacebookBaseChannelHelper
{
	protected $instantArticlesClient = null;

	/**
	 * getApiInstance.
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->facebook)
		{
			$fbChannelAccessToken = $this->get('fbchannel_access_token');
			$fbChannelId = $this->getFbChannelId();

			$this->facebook = new \XTP_BUILD\Facebook\Facebook(
				[
					'app_id' => $this->get('app_id'),
					'app_secret' => $this->get('secret'),
					'default_access_token' => $fbChannelAccessToken,
					'default_graph_version' => 'v2.10'
				]
			);

			$updatedAccessToken = $this->matchToken($fbChannelAccessToken, $fbChannelId);

			if ($updatedAccessToken != $fbChannelAccessToken)
			{
				$this->facebook = new \XTP_BUILD\Facebook\Facebook(
					[
						'app_id' => $this->get('app_id'),
						'app_secret' => $this->get('secret'),
						'default_access_token' => $updatedAccessToken,
						'default_graph_version' => 'v2.10'
					]
				);
			}

			$this->instantArticlesClient = new \XTP_BUILD\Facebook\InstantArticles\Client\Client(
				$this->facebook,
				$fbChannelId,

				// Development mode
				(bool) $this->channel->params->get('fbinstant_dev')
			);
		}

		return $this->facebook;
	}

	/**
	 * sendMessage.
	 *
	 * @param   string  $message  Params
	 * @param   object  $data     Params
	 *
	 * @return	boolean
	 */
	public function sendMessage($message, $data)
	{
		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, 'sendFacebookInstantMessage', $message);

		// Simulated
		if ($this->channel->params->get('use_own_api') == 0)
		{
			return $this->testMessage();
		}

		try
		{
			$title = $data->title;
			$text = $data->fulltext;
			$url = $data->url;
			$originalUrl = $data->org_url;
			$imageUrl = $data->image_url;
			$mediaMode = $this->getMediaMode();

			$nativeObject = $data->xtform->get('native_object');

			if ( (!isset($nativeObject->title)) && (!isset($nativeObject->introtext)) )
			{
				return array(
					true,
					'No enough text for an Instant Article.'
				);
			}

			$this->getApiInstance();

			$transformer = new \XTP_BUILD\Facebook\InstantArticles\Transformer\Transformer;
			$rules = file_get_contents(JPATH_AUTOTWEET_HELPERS . "/channels/facebookinstant_rules.json", true);
			$transformer->loadRules($rules);

			$html = '<!DOCTYPE html><body><h1>' . $title . '</h1>' . $nativeObject->introtext . '</body></html>';
			$document = new \DOMDocument;
			$document->loadHTML($html);

			$header = XTP_BUILD\Facebook\InstantArticles\Elements\Header::create();
			$header->withTitle($title);

			if ( ($mediaMode != 'message') && (!empty($imageUrl)) )
			{
				$header->withCover(
					XTP_BUILD\Facebook\InstantArticles\Elements\Image::create()
						->withURL($imageUrl)
						->withCaption(
							XTP_BUILD\Facebook\InstantArticles\Elements\Caption::create()->appendText($message)
						)
				);
			}

			// Article initialization
			$instantArticle = XTP_BUILD\Facebook\InstantArticles\Elements\InstantArticle::create();
			$instantArticle->withHeader($header);

			// CanonicalUrl
			$instantArticle->withCanonicalUrl($originalUrl);

			// Credits
			$sitename = JFactory::getConfig()->get('sitename');
			$instantArticle->withFooter(
				XTP_BUILD\Facebook\InstantArticles\Elements\Footer::create()->withCredits($sitename)
			);

			$transformer->transform($instantArticle, $document);

			$warnings = $transformer->getWarnings();

			if (!empty($warnings))
			{
				foreach ($warnings as $warning)
				{
					$logger->log(JLog::INFO, 'InstantArticle warning: ' . $warning);
				}
			}

			$this->instantArticlesClient->importArticle(
				$instantArticle,
				(bool) $this->channel->params->get('fbinstant_dev')
			);
			$result = $this->facebook->getLastResponse();
			$decodedBody = $result->getDecodedBody();

			$response = $this->callApiGet('/' . $decodedBody['id']);
			$graphNode = $response->asArray();

			if ($graphNode['status'] == 'FAILED')
			{
				$errors = $graphNode['errors'];
				$error = $errors[0];

				$result = array(
					false,
					$error['level'] . ' - ' . $error['message']
				);
			}
			else
			{
				$msg = 'Facebook id: ' . $decodedBody['id'] . ' - ' . $graphNode['status'];
				$result = array(
					true,
					$msg
				);
			}
		}
		catch (Exception $e)
		{
			$result = array(
				false,
				$e->getCode() . ' - ' . $e->getMessage()
			);
		}

		return $result;
	}
}
