<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (LiOAuth2, Facebook, LinkedIn, etc).
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

/**
 * LiOAuth2CompanyChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class LiOAuth2CompanyChannelHelper extends LiOAuth2ChannelHelper
{
	/**
	 * getAuthorizationUrl
	 *
	 * @return	string
	 */
	public function getAuthorizationUrl()
	{
		if ($this->isAPIv1()) {
			return parent::getAuthorizationUrl();
		}

		return $this->getAuthorizationUrlInternal(self::API2_COMPANY_PERMS);
	}

	/**
	 * getMyCompanies.
	 *
	 * @return	object
	 */
	public function getMyCompanies()
	{
		if ($this->isAPIv1()) {
			return $this->getMyCompaniesv1();
		}

		return $this->getMyCompaniesv2();
	}

	/**
	 * getMyCompanies.
	 *
	 * @return	object
	 */
	public function getMyCompaniesv1()
	{
		$result = null;

		try
		{
			$api = $this->getApiInstance();
			$response = $api->api('GET', '/v1/companies/?is-company-admin=true');

			if (isset($response['_total']))
			{
				if ($response['_total'] == 0)
				{
					return array(false, JText::_('COM_AUTOTWEET_CHANNEL_LINKEDIN_ERR_NOCOMPANIES'));
				}

				if (isset($response['values']))
					{
					$companies = $response['values'];

					if (empty($companies))
					{
						return array();
					}

					$result = array();

					foreach ($companies as $c)
					{
						$company = json_decode(json_encode($c));

						$url = 'https://www.linkedin.com/company/' . $c['id'];
						$company->url = $url;

						$result[] = $company;
					}

					return $result;
				}
				else
				{
					$msg = $response['message'] . ' - ' . $response['status'];
					$result = array(false, $msg);
				}
			}
			else
			{
				$msg = $response['message'] . ' - ' . $response['status'];
				$result = array(false, $msg);
			}
		}
		catch (Exception $e)
		{
			$result = array('id' => false, 'name' => $e->getMessage());
		}

		return $result;
	}

	/**
	 * getMyCompanies.
	 *
	 * @return	object
	 */
	public function getMyCompaniesv2()
	{
		$result = null;

		try
		{
			$api = $this->getLinkedInAPIv2();
			$response = $api->api('GET', '/v2/organizationalEntityAcls?q=roleAssignee&count=100');

			if (isset($response['message']))
			{
				return $this->processResponse($response);
			}

			return $this->getAdministratedCompaniesv2($response);
		}
		catch (Exception $e)
		{
			$result = array('id' => false, 'name' => $e->getMessage());
		}

		return $result;
	}

	protected function getAdministratedCompaniesv2($response)
	{
		$elements = $response['elements'];
		$companies = [];

		foreach ($elements as $company) {
			$isAdministrated = ($company['role'] === "ADMINISTRATOR") && ($company['state'] === "APPROVED");

			if (!$isAdministrated) {
				continue;
			}

			$item = $this->getCompanyv2($company['organizationalTarget']);

			if (($item) && (is_object($item))) {
				$companies[] = $item;
			}
		}

		return $companies;
	}

	protected function getCompanyv2($organizationId)
	{
		$api = $this->getLinkedInAPIv2();
		list($url, $li, $org, $id) = explode(':', $organizationId);

		$response = $api->api('GET', '/v2/organizations/'.$id);

		if (isset($response['message'])) {
			return $this->processResponse($response);
		}

		if (empty($response)) {
			return null;
		}

		$company = new StdClass;
		$company->id = $id;
		$company->name = $response['localizedName'];
		$company->url = 'https://www.linkedin.com/company/' . $response['vanityName'];

		return $company;
	}

	/**
	 * sendMessage.
	 *
	 * @param   string  $message  Params
	 * @param   object  $data     Params
	 *
	 * @return  boolean
	 */
	public function sendMessageAPIv1($message, $data)
	{
		$result = array(
			false,
			'Unknown error'
		);

		$title = $data->title;
		$text = $data->fulltext;
		$url = $data->url;
		$image_url = $data->image_url;
		$media_mode = $this->getMediaMode();

		$result = null;
		$content = array();

		// Post message and/or media
		switch ($media_mode)
		{
			case 'attachment':
				$post_attach = true;
				$post_msg = false;
				break;
			case 'both':
				$post_msg = true;
				$post_attach = true;
				break;
			case 'message':
			default:
				$post_msg = true;
				$post_attach = false;
		}

		// Media: do also not post when text and image are empty
		if (($post_attach) && (!empty($title)) && (!empty($url)))
		{
			// Prepare content
			$content['content']['title'] = TextUtil::truncString($title, self::MAX_CHARS_TITLE);
			$content['content']['submitted-url'] = $url;
			$content['content']['submitted-image-url'] = $image_url;

			// Strlen shorter than JString::strlen for UTF-8  - 2 char languages E.g. Hebrew
			$text = TextUtil::truncString($text, self::MAX_CHARS_DESC);
			$content['content']['description'] = $text;
		}

		// Message
		if ($post_msg)
		{
			$content['comment'] = TextUtil::truncString($title, self::MAX_CHARS_COMMENT);
		}

		$content['visibility'] = array(
			'code' => 'anyone'
		);

		try
		{
			$companyId = $this->get('company_id');

			if (empty($companyId))
			{
				throw new Exception('Invalid Company. Please, select a Company.');
			}

			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::INFO, 'LiOAuth2CompanyChannelHelper sendMessage company_id=' . $companyId, $content);

			$api = $this->getApiInstance();
			$options = array('json' => $content);
			$response = $api->post('/v1/companies/' . $companyId . '/shares', $options);

			if (isset($response['updateUrl']))
			{
				$result = array(
						true,
						JText::_('COM_AUTOTWEET_OK') . ' - ' . $response['updateUrl']
				);

				return $result;
			}

			if (isset($response['message']))
			{
				$result = array(
						false,
						$response['message'] . ' - ' . $response['status']
				);

				return $result;
			}
		}
		catch (Exception $e)
		{
			$result = array(
					false,
					$e->getMessage()
			);
		}

		return $result;
	}

	protected function sendMessageAPIv2($author, $message, $data)
	{
		$companyId = $this->get('company_id');

		if (empty($companyId))
		{
			throw new Exception('Invalid Company. Please, select a Company.');
		}

		return parent::sendMessageAPIv2($companyId, $message, $data);
	}

	protected function getUrn($id)
	{
		return 'urn:li:company:' . $id;
	}

	protected function getMeAPIv2()
	{
		$api = $this->getLinkedInAPIv2();

		return $api->get('/v2/me?fields=id,firstName,lastName,vanityName');
	}
}
