<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (OneSignal, Facebook, LinkedIn, etc).
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

/**
 * OneSignalChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class OneSignalChannelHelper extends ChannelHelper
{
	protected $oneSignalClient = null;

	protected $appId = null;

	protected $restApiKey = null;

	protected $userAuthKey = null;

	protected $is_auth = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel      Params.
	 * @param   string  $appId        Params.
	 * @param   string  $restApiKey   Params.
	 * @param   string  $userAuthKey  Params.
	 */
	public function __construct($channel, $appId = null, $restApiKey = null, $userAuthKey = null)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->appId = $this->channel->params->get('app_id');
			$this->restApiKey = $this->channel->params->get('rest_api_key');
			$this->userAuthKey = $this->channel->params->get('user_auth_key');
		}

		if ($appId)
		{
			$this->appId = $appId;
			$this->restApiKey = $restApiKey;
			$this->userAuthKey = $userAuthKey;
		}
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->oneSignalClient)
		{
			$config = new \XTP_BUILD\OneSignal\Config;
			$config->setApplicationId($this->appId);
			$config->setApplicationAuthKey($this->restApiKey);
			$config->setUserAuthKey($this->userAuthKey);

			$guzzleMessageFactory = new \XTP_BUILD\Http\Message\MessageFactory\GuzzleMessageFactory;

			$guzzle = new \XTP_BUILD\GuzzleHttp\Client;
			$guzzleAdapter = new \XTP_BUILD\Http\Adapter\Guzzle6\Client($guzzle);

			$client = new \XTP_BUILD\Http\Client\Common\HttpMethodsClient($guzzleAdapter, $guzzleMessageFactory);
			$this->oneSignalClient = new \XTP_BUILD\OneSignal\OneSignal($config, $client);
		}

		return $this->oneSignalClient;
	}

	/**
	 * isAuth()
	 *
	 * @return	bool
	 */
	public function isAuth()
	{
		if (empty($this->appId))
		{
			$this->appId = null;

			return false;
		}

		try
		{
			$this->getApiInstance();
			$devices = $this->oneSignalClient->devices->getAll();

			return true;
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Just in case, it is shown someday
			JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
		}

		return false;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, 'sendOneSignalMessage', $message);

		$isAuth = $this->isAuth();


		if (!$isAuth)
		{
			return array(
				false,
				JText::_('COM_AUTOTWEET_CHANNEL_NOT_AUTH_ERR')
			);
		}

		$result = array(false, 'OneSignal Unknown Error', null);

		try
		{
			$this->getApiInstance();

			$title = JFactory::getConfig()->get('sitename');

			$notification = [
				'contents' => [
					'en' => $message
				],
				'headings' => [
					'en' => $title
				],
				'included_segments' => ['All']
			];

			// Web Push
			if ($this->channel->params->get('chrome'))
			{
				$notification['isChromeWeb'] = true;
			}

			if ($this->channel->params->get('firefox'))
			{
				$notification['isFirefox'] = true;
			}

			if ($this->channel->params->get('safari'))
			{
				$notification['isSafari'] = true;
			}

			// Push Notifications
			if ($this->channel->params->get('ios'))
			{
				$notification['isIos'] = true;
			}

			if ($this->channel->params->get('android'))
			{
				$notification['isAndroid'] = true;
			}

			if ($this->channel->params->get('adm'))
			{
				$notification['isAdm'] = true;
			}

			if ($this->channel->params->get('wp'))
			{
				$notification['isWP'] = true;
				$notification['isWP_WNS'] = true;
			}

			if (!empty($data->org_url))
			{
				$notification['url'] = $data->org_url;
			}

			if (($this->getMediaMode() != 'message') && (!empty($data->image_url)))
			{
				if ($this->channel->params->get('android'))
				{
					$notification['big_picture'] = $data->image_url;
				}

				if ($this->channel->params->get('adm'))
				{
					$notification['adm_big_picture'] = $data->image_url;
				}

				if ($this->channel->params->get('chrome'))
				{
					$notification['chrome_big_picture'] = $data->image_url;
					$notification['chrome_web_image'] = $data->image_url;
				}
			}

			// $logger->log(JLog::INFO, 'sendOneSignalMessage notification:', $notification);

			$response = $this->oneSignalClient->notifications->add($notification);
			$messageId = $response['id'];

			if (empty($messageId))
			{
				$errors = $response['errors'];
				$error = array_pop($errors);

				$result = array(
					false
				);
				$result[] = $error;
			}
			else
			{
				$result = array(
					true
				);
				$result[] = 'OK - ' . $messageId;
			}
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}
}
