<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (PushAlert, Facebook, LinkedIn, etc).
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

/**
 * PushAlertChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class PushAlertChannelHelper extends ChannelHelper
{
	protected $pushAlertClient = null;

	protected $restApiKey = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel      Params.
	 * @param   string  $restApiKey   Params.
	 */
	public function __construct($channel, $restApiKey = null)
	{
		parent::__construct($channel);

		if ($channel->id)
		{
			$this->restApiKey = $this->channel->params->get('rest_api_key');
		}

		if ($restApiKey)
		{
			$this->restApiKey = $restApiKey;
		}
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->pushAlertClient)
		{
			$this->pushAlertClient = XTP_BUILD\Http\Discovery\HttpClientDiscovery::find();
			$this->serviceUri = new \XTP_BUILD\GuzzleHttp\Psr7\Uri('https://api.pushalert.co/rest/v1/send');
		}

		return $this->pushAlertClient;
	}

	/**
	 * isAuth()
	 *
	 * @return	bool
	 */
	public function isAuth()
	{
		if (empty($this->restApiKey))
		{
			$this->restApiKey = null;

			return false;
		}

		try
		{
			$this->getApiInstance();

			$factory = XTP_BUILD\Http\Discovery\MessageFactoryDiscovery::find();
			$request = $factory->createRequest(
				'POST',
				$this->serviceUri,
				[
					'Authorization' => 'api_key=' . $this->restApiKey,
				]
			);

			$response = $this->pushAlertClient->sendRequest($request);
			$httpStatusCode = $response->getStatusCode();

			// HTTP_STATUS_UNAUTHORIZED
			return (!$httpStatusCode != 401);
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			// Just in case, it is shown someday
			JFactory::getApplication()->enqueueMessage($e->getMessage(), 'error');
		}

		return false;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	array
	 */
	public function sendMessage($message, $data)
	{
		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, 'sendPushAlertMessage', $message);

		$result = array(false, 'PushAlert Unknown Error', null);

		try
		{
			$this->getApiInstance();

			$title = JFactory::getConfig()->get('sitename');

			$postVars = [
				"title" => $title,
				"message" => $message,
				"url" => $data->org_url
			];

			$factory = XTP_BUILD\Http\Discovery\MessageFactoryDiscovery::find();
			$request = $factory->createRequest(
				'POST',
				$this->serviceUri,
				[
					'Authorization' => 'api_key=' . $this->restApiKey,
				],
				http_build_query($postVars)
			);

			$response = $this->pushAlertClient->sendRequest($request);

			$httpStatusCode = $response->getStatusCode();
			$body = (string) $response->getBody();
			$data = json_decode($body, true);

			if ($httpStatusCode == 200)
			{
				$messageId = $data['id'];

				$result = array(
					true
				);
				$result[] = 'OK - ' . $messageId;
			}
			else
			{
				$result = array(
					false
				);
				$result[] = $data['msg'];
			}
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return $result;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}
}
