<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

/**
 * TwAppHelper class.
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class TwAppHelper
{
	private $twitter = null;

	protected $consumer_key = null;

	protected $consumer_secret = null;

	protected $access_token = null;

	protected $access_token_secret = null;

	/**
	 * TwAppHelper.
	 *
	 * @param   string  $consumer_key         Params.
	 * @param   string  $consumer_secret      Params.
	 * @param   string  $access_token         Params.
	 * @param   string  $access_token_secret  Params.
	 */
	public function __construct($consumer_key = null, $consumer_secret = null, $access_token = null, $access_token_secret = null)
	{
		$this->consumer_key = $consumer_key;
		$this->consumer_secret = $consumer_secret;
		$this->access_token = $access_token;
		$this->access_token_secret = $access_token_secret;
	}

	/**
	 * login.
	 *
	 * @return	object
	 */
	public function login()
	{
		if (!$this->twitter)
		{
			$this->twitter = new \XTP_BUILD\Abraham\TwitterOAuth\TwitterOAuth(
				$this->consumer_key,
				$this->consumer_secret,
				$this->access_token,
				$this->access_token_secret
			);
		}

		return $this->twitter;
	}

	/**
	 * getApi.
	 *
	 * @return	object
	 */
	public function getApi()
	{
		return $this->twitter;
	}

	/**
	 * verify.
	 *
	 * @return	boolean
	 */
	public function verify()
	{
		$connection = $this->login();
		$response = $connection->get('account/verify_credentials');

		if ($connection->getLastHttpCode() == 200)
		{
			$user = $response;
			$url = 'https://twitter.com/' . $user->screen_name;

			$message = array(
				'status' => true,
				'error_message' => null,
				'user' => $user,
				'url' => $url
			);
		}
		else
		{
			if (isset($response->errors))
			{
				$error = array_shift($response->errors);
				$message = $error->code . ' - ' . $error->message;
			}
			else
			{
				$message = 'Error ' . $connection->getLastHttpCode();
			}

			$message = array(
				'status' => false,
				'error_message' => $message
			);
		}

		return $message;
	}

	/**
	 * checkTimestamp.
	 *
	 * @return	boolean
	 */
	public static function checkTimestamp()
	{
		// Get component parameter - Offline mode
		$version_check = EParameter::getComponentParam(CAUTOTWEETNG, 'version_check', 1);

		if (!$version_check)
		{
			return '998 Offline';
		}

		$appHelper = new TwAppHelper;
		$appHelper->verify();
		$connection = $appHelper->getApi();
		$response = $connection->response;

		$dateCompare = '999 Unable to check';

		if ( (isset($connection->response->headers))
			&& (array_key_exists('date', $connection->response->headers)) )
		{
			$headers = $connection->response->headers;
			$twitterDate = $headers['date'];
			$twistamp = strtotime($twitterDate);
			$srvstamp = time();
			$dateCompare = abs($srvstamp - $twistamp);
		}

		return $dateCompare;
	}

	/**
	 * getUserTimeline.
	 *
	 * @param   string  $twUsername   Param
	 * @param   int     $twMaxTweets  Param
	 *
	 * @return	array
	 */
	public function getUserTimeline($twUsername, $twMaxTweets)
	{
		$connection = $this->login();
		$response = $connection->get(
			'statuses/user_timeline',
			[
						'screen_name' => $twUsername,
						'count' => $twMaxTweets
			]
		);

		return $response;
	}

	/**
	 * Obtain a request token from Twitter
	 *
	 * @return string
	 */
	public function getRequestTokenUrl()
	{
		$input = new F0FInput;
		$itemid = $input->getInt('Itemid');

		$oauth_callback = 'index.php?option=com_autotweet&view=userchannels';

		if ($itemid)
		{
			$oauth_callback .= '&Itemid=' . $itemid;
		}

		$oauth_callback = RouteHelp::getInstance()->getAbsoluteUrl($oauth_callback);

		// Send request for a request token
		$connection = $this->twitter;
		$response = $connection->oauth('oauth/request_token');

		if ($connection->getLastHttpCode() == 200)
		{
			$session = JFactory::getSession();

			// State is now 1
			$session->set('twitter-authstate', 1);

			// Redirect the user to Twitter to authorize
			$url = 'https://twitter.com/oauth/authorize?oauth_token=' . $response['oauth_token'];

			return $url;
		}
		else
		{
			if (isset($response->errors))
			{
				$error = array_shift($response->errors);
				$message = $error->code . ' - ' . $error->message;
			}
			else
			{
				$message = 'Error ' . $connection->getLastHttpCode();
			}

			throw new Exception($message);
		}

		return false;
	}

	/**
	 * Obtain a request token from Twitter
	 *
	 * @return string
	 */
	public function getAccessToken()
	{
		$session = JFactory::getSession();

		$input = new F0FInput;
		$oauth_token = $input->get('oauth_token');
		$oauth_verifier = $input->get('oauth_verifier');

		$this->access_token = $oauth_token;
		$this->token_secret = $oauth_verifier;
		$this->login();

		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, 'getAccessToken - oauth_verifier: ' . $oauth_verifier);

		// Send request for an access token
		$connection = $this->twitter;
		$response = $connection->oauth(
			'oauth/access_token',
			[
				// Pass the oauth_verifier received from Twitter
				'oauth_token' => $oauth_token,
				'oauth_verifier' => $oauth_verifier
			]
		);

		if ($connection->getLastHttpCode() == 200)
		{
			$access_token = $response['oauth_token'];
			$access_token_secret = $response['oauth_token_secret'];

			return array(
				'access_token' => $access_token,
				'access_token_secret' => $access_token_secret
			);
		}
		else
		{
			if (isset($response->errors))
			{
				$error = array_shift($response->errors);
				$message = $error->code . ' - ' . $error->message;
			}
			else
			{
				$message = 'Error ' . $connection->getLastHttpCode();
			}

			throw new Exception($message);
		}

		return false;
	}
}
