<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - AutoTweetNG posts content to social channels (Vk, Facebook, LinkedIn, etc).
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

use XTP_BUILD\VK\Client\VKApiClient;
use XTP_BUILD\VK\Client\Enums\VKLanguage;

/**
 * VkChannelHelper
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class VkChannelHelper extends ChannelHelper
{
	const APP_SCOPE = 'groups,offline,photos,wall';

	const APP_VERSION = '5.80';

	const AUTHORIZE_URL_FORMAT = 'https://oauth.vk.com/authorize?client_id=%s&scope=%s&v=%s&response_type=token&redirect_uri=https://oauth.vk.com/blank.html';

	const POST_URL_FORMAT = 'https://vk.com/wall%s_%s';

	protected $vk = null;

	protected $access_token = null;

	protected $is_auth = null;

	protected $userId = null;

	protected $user = null;

	/**
	 * ChannelHelper.
	 *
	 * @param   object  $channel  Params.
	 */
	public function __construct($channel)
	{
		parent::__construct($channel);

		$this->access_token = $this->get('access_token');

		if (($this->access_token) && (is_string($this->access_token)))
		{
			$this->access_token = json_decode($this->access_token);
		}
	}

	/**
	 *
	 *
	 * @param   int     $clientAppId  Param
	 *
	 * @return	string
	 */
	public function getAuthorizationUrl($clientAppId)
	{
		return sprintf(
			self::AUTHORIZE_URL_FORMAT,
			$clientAppId,
			self::APP_SCOPE,
			self::APP_VERSION
		);
	}

	/**
	 * getUserId
	 *
	 * @return	int
	 */
	public function getUserId()
	{
		if ($this->userId)
		{
			return $this->userId;
		}

		if (!$this->isAuth())
		{
			return null;
		}

		return $this->userId;
	}

	/**
	 * isAuth
	 *
	 * @return	bool
	 */
	public function isAuth()
	{
		if (empty($this->access_token))
		{
			return false;
		}

		try
		{
			$vk = $this->getApiInstance();

			$profileInfo = $vk->account()->getProfileInfo($this->getAccessToken());

			$users = $vk->users()->get($this->getAccessToken());
			$user = array_pop($users);
			$this->userId = $user['id'];

			$this->user = $profileInfo;

			return true;
		}
		catch (Exception $e)
		{
			$this->_access_token = null;

			// Invalidating access_token
			if ((isset($this->channel->id)) && ($this->channel->id))
			{
				$ch = F0FTable::getAnInstance('Channel', 'AutoTweetTable');
				$result = $ch->load($this->channel->id);

				if (!$result)
				{
					throw new Exception(JText::_('COM_AUTOTWEET_CHANNEL_NOTLOADED'));
				}

				$params = $ch->params;
				$registry = new JRegistry;
				$registry->loadString($params);
				$registry->set('access_token', null);
				$ch->bind(array('params' => (string) $registry));
				$ch->store();
			}

			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());

			return false;
		}
	}

	/**
	 * getUserSettings()
	 *
	 * @return	object
	 */
	public function getUserSettings()
	{
		$vk = $this->getApiInstance();

		if ($this->isAuth()) {
			$url = 'https://vk.com/' . $this->user['screen_name'] ;

			$message = array(
				'status' => true,
				'error_message' => null,
				'user' => $userId,
				'url' => $url
			);

			return $message;
		}

		$message = array(
			'status' => false,
			'error_message' => 'Not Authorized!',
			'user' => null,
			'url' => null
		);

		return $message;
	}

	/**
	 * Internal service functions
	 *
	 * @return	object
	 */
	protected function getApiInstance()
	{
		if (!$this->vk)
		{
			$this->vk = new VKApiClient(self::APP_VERSION, VKLanguage::ENGLISH);
		}

		return $this->vk;
	}

	/**
	 * sendMessage
	 *
	 * @param   string  $message  Param
	 * @param   array   $data     Param
	 *
	 * @return	bool
	 */
	public function sendMessage($message, $data)
	{
		try
		{
			$image_url = $data->image_url;
			$media_mode = $this->getMediaMode();

			if (($media_mode != 'message') && (!empty($image_url)))
			{
				return $this->_sendVkMessageWithImage($message, $image_url, $data->url);
			}

			return $this->_sendVkMessage($message, $image_url);
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}
	}

	/**
	 * _sendVkMessage
	 *
	 * @param   string  $status_msg  Param
	 * @param   string  $image_url   Param
	 *
	 * @return	array
	 */
	protected function _sendVkMessage($status_msg, $image_url)
	{
		$logger = AutotweetLogger::getInstance();
		$logger->log(JLog::INFO, '_sendVkMessage', $status_msg);

		$vk = $this->getApiInstance();

		try
		{
			$userId = $this->getUserId();

			if (!$userId)
			{
				return array(
					false,
					JText::_('COM_AUTOTWEET_CHANNEL_NOT_AUTH_ERR')
				);
			}

			$gid = $this->channel->params->get('vkgroup_id');
			$ownerId = $gid ? -1 * $gid : $userId;

			$parameters = array(
				'owner_id' => $ownerId,
				'message' => $status_msg,
				'from_group' => ($gid ? 1 : 0)
			);
			$response = $vk->wall()->post($this->getAccessToken(), $parameters);

			$postUrl = $this->getPostUrl($ownerId, $response['post_id']);

			return array(
				true,
				'OK - ' . $postUrl
			);
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return array(
			false,
			'Failed _sendVkMessage'
		);
	}

	/**
	 * uploadImage
	 *
	 * @param   string  $image_url  Param
	 *
	 * @return	object
	 */
	public function uploadImage($image_url)
	{
		if (empty($image_url))
		{
			return false;
		}

		$imagefile = ImageUtil::getInstance()->downloadImage($image_url);

		if (!$imagefile)
		{
			return false;
		}

		$status = false;

		try
		{
			$uid = $this->getUserId();
			$gid = $this->channel->params->get('vkgroup_id');

			$vk = $this->getApiInstance();
			$address = $vk->photos()->getWallUploadServer($this->getAccessToken());
			$photo = $vk->getRequest()->upload($address['upload_url'], 'photo', $imagefile);

			$parameters = array(
				'server' => $photo['server'],
				'photo' => $photo['photo'],
				'hash' => $photo['hash'],
			);

			if ($gid)
			{
				$parameters['gid'] = $gid;
			}
			else
			{
				$parameters['uid'] = $uid;
			}

			$statuses = $vk->photos()->saveWallPhoto($this->getAccessToken(), $parameters);
			$status = array_pop($statuses);
		}
		catch (Exception $e)
		{
			$logger = AutotweetLogger::getInstance();
			$logger->log(JLog::ERROR, $e->getMessage());
		}

		ImageUtil::getInstance()->releaseImage($imagefile);

		return $status;
	}

	protected function getPostUrl($userId, $postId)
	{
		return sprintf(
			self::POST_URL_FORMAT,
			$userId,
			$postId
		);
	}

	/**
	 * _sendVkMessageWithImage
	 *
	 * @param   string  $status_msg  Param
	 * @param   string  $image_url   Param
	 * @param   string  $url         Param
	 *
	 * @return	array
	 */
	protected function _sendVkMessageWithImage($status_msg, $image_url, $url)
	{
		$vk = $this->getApiInstance();

		try
		{
			$userId = $this->getUserId();

			if (!$userId)
			{
				return array(
					false,
					JText::_('COM_AUTOTWEET_CHANNEL_NOT_AUTH_ERR')
				);
			}

			$result = $this->uploadImage($image_url);

			if (!$result)
			{
				return array(
					false,
					'Failed to upload!'
				);
			}

			$photo_param = $result['id'] . ',' . $url;

			$gid = $this->channel->params->get('vkgroup_id');
			$ownerId = $gid ? -1 * $gid : $userId;

			$parameters = array(
				'owner_id' => $ownerId,
				'message' => $status_msg,
				'from_group' => ($gid ? 1 : 0),
				'attachments' => $photo_param
			);
			$response = $vk->wall()->post($this->getAccessToken(), $parameters);

			$postUrl = $this->getPostUrl($ownerId, $response['post_id']);

			return array(
				true,
				'OK - ' . $postUrl
			);
		}
		catch (Exception $e)
		{
			return array(
				false,
				$e->getMessage()
			);
		}

		return array(
			false,
			'Failed _sendVkMessageWithImage'
		);
	}

	/**
	 * hasWeight
	 *
	 * @return	bool
	 */
	public function hasWeight()
	{
		return true;
	}

	/**
	 * includeHashTags
	 *
	 * @return  bool
	 */
	public function includeHashTags()
	{
		return $this->channel->params->get('hashtags', true);
	}

	/**
	 * getGroups()
	 *
	 * @return	array
	 */
	public function getGroups()
	{
		$vk = $this->getApiInstance();

		$parameters = array(
			'extended' => 1,
			'filter' => 'moder'
		);
		$response = $vk->groups()->get($this->getAccessToken(), $parameters);

		$items = $this->prepareGroupItems($response);

		return array(
			'status' => true,
			'error_message' => null,
			'items' => $items
		);
	}

	public function getJsonAccessToken()
	{
		return $this->get('access_token');
	}

	public function getAccessToken()
	{
		if ($this->access_token)
		{
			return $this->access_token->access_token;
		}

		return null;
	}

	public function getExpiresIn()
	{
		return 0;
	}

	protected function prepareGroupItems($response)
	{
		$items = [];

		$groups = $response['items'];

		if (empty($groups))
		{
			return [];
		}

		foreach ($groups as $group)
		{
			$group['gid'] = $group['id'];
			$group['url'] = 'https://vk.com/' . $group['screen_name'];
			$items[] = $group;
		}

		return $items;
	}
}
