<?php

/**
 * @package     Extly.Components
 * @subpackage  com_autotweet - A powerful social content platform to manage multiple social networks.
 *
 * @author      Extly, CB. <team@extly.com>
 * @copyright   Copyright (c)2007-2019 Extly, CB. All rights reserved.
 * @license     http://https://www.gnu.org/licenses/gpl-3.0.html GNU/GPL
 * @link        https://www.extly.com
 */

defined('_JEXEC') or die;

/**
 * AutotweetModelFeedContents
 *
 * @package     Extly.Components
 * @subpackage  com_autotweet
 * @since       1.0
 */
class AutotweetModelFeedContent extends F0FModel
{
	/**
	 * Binds the data to the model and tries to save it
	 *
	 * @param   array|object  $data  The source data array or object
	 *
	 * @return  boolean  True on success
	 */
	public function save($data)
	{
		$this->_savestate = false;

		// Images
		$this->_loadImages($data);

		// Urls
		$this->_loadUrls($data);

		// Saving
		$status = parent::save($data);

		return $status;
	}

	/**
	 * This method runs after the data is saved to the $table.
	 *
	 * @param   F0FTable  &$table  The table which was saved
	 *
	 * @return  boolean
	 */
	protected function onAfterSave(&$table)
	{
		$error = $this->getError();

		if ($error)
		{
			throw new Exception($error);
		}

		return true;
	}

	/**
	 * A method for getting the form from the model.
	 *
	 * @param   array    $data      Data for the form.
	 * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
	 * @param   boolean  $source    The name of the form. If not set we'll try the form_name state variable or fall back to default.
	 *
	 * @return  mixed  A F0FForm object on success, false on failure
	 */
	public function getForm($data = array(), $loadData = true, $source = null)
	{
		$f0fPlatform = F0FPlatform::getInstance();
		$isFrontend = $f0fPlatform->isFrontend();

		$this->input->set('option', 'com_content');
		$this->input->set('view', $isFrontend ? 'form' : 'article');

		$form = parent::getForm($data, $loadData, $source);

		return $form;
	}

	/**
	 * Method to toggle the featured setting of articles.
	 *
	 * @param   array    $pks    The ids of the items to toggle.
	 * @param   integer  $value  The value to toggle to.
	 *
	 * @return  boolean  True on success.
	 */
	public static function featured($pks, $value = 0)
	{
		// Sanitize the ids.
		$pks = (array) $pks;
		JArrayHelper::toInteger($pks);

		if (empty($pks))
		{
			throw new Exception(JText::_('COM_CONTENT_NO_ITEM_SELECTED'));
		}

		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->update($db->quoteName('#__content'))
			->set('featured = ' . (int) $value)
			->where('id IN (' . implode(',', $pks) . ')');
		$db->setQuery($query);
		$db->execute();

		if ((int) $value == 0)
		{
			// Adjust the mapping table.
			// Clear the existing features settings.
			$query = $db->getQuery(true)
				->delete($db->quoteName('#__content_frontpage'))
				->where('content_id IN (' . implode(',', $pks) . ')');
			$db->setQuery($query);
			$db->execute();
		}
		else
		{
			// First, we find out which of our new featured articles are already featured.

			$query = $db->getQuery(true)
				->select('f.content_id')
				->from('#__content_frontpage AS f')
				->where('content_id IN (' . implode(',', $pks) . ')');

			$db->setQuery($query);

			$old_featured = $db->loadColumn();

			// We diff the arrays to get a list of the articles that are newly featured

			$new_featured = array_diff($pks, $old_featured);

			// Featuring.
			$tuples = array();

			foreach ($new_featured as $pk)
			{
				$tuples[] = $pk . ', 0';
			}

			if (count($tuples))
			{
				$db = JFactory::getDbo();
				$columns = array('content_id', 'ordering');
				$query = $db->getQuery(true)
					->insert($db->quoteName('#__content_frontpage'))
					->columns($db->quoteName($columns))
					->values($tuples);
				$db->setQuery($query);
				$db->execute();
			}
		}

		self::_reorder();
		self::_cleanCache();

		return true;
	}

	/**
	 * reorder
	 *
	 * @return  void
	 */
	private static function _reorder()
	{
		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_content/tables');
		$table = JTable::getInstance('Featured', 'ContentTable');
		$table->reorder();
	}

	/**
	 * Custom clean the cache of com_content and content modules
	 *
	 * @return  void
	 */
	private static function _cleanCache()
	{
		self::_cleanCacheGroup('com_content');
		self::_cleanCacheGroup('mod_articles_archive');
		self::_cleanCacheGroup('mod_articles_categories');
		self::_cleanCacheGroup('mod_articles_category');
		self::_cleanCacheGroup('mod_articles_latest');
		self::_cleanCacheGroup('mod_articles_news');
		self::_cleanCacheGroup('mod_articles_popular');
	}

	/**
	 * _loadImages
	 *
	 * @param   array  &$data  Param
	 *
	 * @return  void
	 */
	private function _loadImages(&$data)
	{
		if (isset($data['images']) && is_array($data['images']))
		{
			$registry = new JRegistry;
			$registry->loadArray($data['images']);
			$data['images'] = (string) $registry;
		}
	}

	/**
	 * _loadImages
	 *
	 * @param   array  &$data  Param
	 *
	 * @return  void
	 */
	private function _loadUrls(&$data)
	{
		if (isset($data['urls']) && is_array($data['urls']))
		{
			foreach ($data['urls'] as $i => $url)
			{
				if ($url != false && ($i == 'urla' || $i == 'urlb' || $i = 'urlc'))
				{
					if (class_exists('JStringPunycode'))
					{
						$data['urls'][$i] = JStringPunycode::urlToPunycode($url);
					}
					else
					{
						$data['urls'][$i] = $url;
					}
				}
			}

			$registry = new JRegistry;
			$registry->loadArray($data['urls']);
			$data['urls'] = (string) $registry;
		}
	}

	/**
	 * Clean the cache
	 *
	 * @param   string   $group      The cache group
	 * @param   integer  $client_id  The ID of the client
	 *
	 * @return  void
	 */
	private static function _cleanCacheGroup($group, $client_id = 0)
	{
		$conf = JFactory::getConfig();

		$options = array(
						'defaultgroup' => $group,
						'cachebase' => ($client_id) ? JPATH_ADMINISTRATOR . '/cache' : $conf->get('cache_path', JPATH_SITE . '/cache'));

		$cache = JCache::getInstance('callback', $options);
		$cache->clean();
	}
}
